@@ -7,7 +7,11 @@ import scanpy as sc
 import itertools
 
 from sklearn.decomposition import PCA
-from scipy.spatial.qhull import _Qhull
+
+try:
+    from scipy.spatial.qhull import _Qhull
+except:
+    from scipy.spatial.qhull._qhull import _Qhull
 from shapely.geometry import Point, Polygon, MultiLineString, LineString
 from shapely.geometry.multipolygon import MultiPolygon
 from sklearn.neighbors import NearestNeighbors
@@ -39,7 +43,9 @@ def _isBetween(a, b, c):
     if dotproduct < 0:
         return False
 
-    squaredlengthba = (b[0] - a[0]) * (b[0] - a[0]) + (b[1] - a[1]) * (b[1] - a[1])
+    squaredlengthba = (b[0] - a[0]) * (b[0] - a[0]) + (b[1] - a[1]) * (
+        b[1] - a[1]
+    )
     if dotproduct > squaredlengthba:
         return False
     return True
@@ -107,7 +113,9 @@ def remove_intersections(nodep, edges):
             if line1.intersects(line2):
                 if list(np.array(line1.intersection(line2))) not in lnodep:
                     new_nodep = np.append(
-                        new_nodep, np.array(line1.intersection(line2))[None], axis=0
+                        new_nodep,
+                        np.array(line1.intersection(line2))[None],
+                        axis=0,
                     )
                     intersects_idx = [list(new_edges[i]), list(new_edges[j])]
                     new_edges.pop(new_edges.index(intersects_idx[0]))
@@ -117,7 +125,9 @@ def remove_intersections(nodep, edges):
                         new_edges.append([n, len(new_nodep) - 1])
                     break
 
-        multiline = MultiLineString([LineString(new_nodep[e]) for e in new_edges])
+        multiline = MultiLineString(
+            [LineString(new_nodep[e]) for e in new_edges]
+        )
         lnodep = new_nodep.tolist()
     return new_nodep, np.array(new_edges)
 
@@ -279,7 +289,10 @@ def addLoops(
     leaves = [k for k, v in epg.degree if v == 1]
     edge_lengths = np.sqrt(
         np.sum(
-            (init_nodes_pos[init_edges[:, 0], :] - init_nodes_pos[init_edges[:, 1], :])
+            (
+                init_nodes_pos[init_edges[:, 0], :]
+                - init_nodes_pos[init_edges[:, 1], :]
+            )
             ** 2,
             axis=1,
         )
@@ -301,7 +314,10 @@ def addLoops(
 
     if verbose:
         print(
-            f"Using default parameters: max_n_points={max_n_points}, radius={radius:.2f}, min_node_n_points={min_node_n_points},min_path_len={min_path_len}, nnodes={nnodes}"
+            f"Using default parameters: max_n_points={max_n_points},"
+            f" radius={radius:.2f},"
+            f" min_node_n_points={min_node_n_points},min_path_len={min_path_len},"
+            f" nnodes={nnodes}"
         )
 
     # --- Get candidate nodes to connect
@@ -327,7 +343,9 @@ def addLoops(
         root_branch = [b for b in branches if leaves[i] in b][0]
 
         if allow_same_branch:
-            _cand_nodes = [node for b in branches for node in b if node in ind[i]]
+            _cand_nodes = [
+                node for b in branches for node in b if node in ind[i]
+            ]
         else:
             _cand_nodes = [
                 node
@@ -361,7 +379,9 @@ def addLoops(
         for c in candidate_nodes[i]:
 
             clus = (part == c) | (part == l)
-            X_fit = np.vstack((init_nodes_pos[c], init_nodes_pos[l], X[clus.flat]))
+            X_fit = np.vstack(
+                (init_nodes_pos[c], init_nodes_pos[l], X[clus.flat])
+            )
             try:
                 pg = elpigraph.computeElasticPrincipalCurve(
                     X_fit,
@@ -379,7 +399,8 @@ def addLoops(
                 loop_leaves.append(np.inf)
                 # candidate curve has infinite energy, ignore error
                 if e.args == (
-                    "local variable 'NewNodePositions' referenced before assignment",
+                    "local variable 'NewNodePositions' referenced before"
+                    " assignment",
                 ):
                     continue
                 else:
@@ -443,7 +464,9 @@ def addLoops(
                 cycle_points = np.isin(cent_part.flat, cycle_edges)
 
                 if X.shape[1] > 2:
-                    pca = PCA(n_components=2, svd_solver="arpack").fit(X[cycle_points])
+                    pca = PCA(n_components=2, svd_solver="arpack").fit(
+                        X[cycle_points]
+                    )
                     X_cycle_2d = pca.transform(X[cycle_points])
                     cycle_2d = pca.transform(cycle_nodep)
                 else:
@@ -472,13 +495,19 @@ def addLoops(
                     if type(shrunk_cycle_2d) == MultiPolygon:
                         in_shrunk_cycle = np.ones(len(X_inside), dtype=bool)
                     else:
-                        shrunk_cycle_2d = np.array(shrunk_cycle_2d.exterior.coords)
+                        shrunk_cycle_2d = np.array(
+                            shrunk_cycle_2d.exterior.coords
+                        )
 
                         # prevent bug when self-intersection
                         if len(shrunk_cycle_2d) == 0:
-                            in_shrunk_cycle = np.ones(len(X_inside), dtype=bool)
+                            in_shrunk_cycle = np.ones(
+                                len(X_inside), dtype=bool
+                            )
                         else:
-                            in_shrunk_cycle = in_hull(shrunk_cycle_2d, X_inside)
+                            in_shrunk_cycle = in_hull(
+                                shrunk_cycle_2d, X_inside
+                            )
                     idx_close = in_shrunk_cycle | (w < 1)
                     w = 1 - w / w.max()
                     w[idx_close] = 1
@@ -492,7 +521,10 @@ def addLoops(
                 if init_nodes_pos.shape[1] == 2:
                     intersect = not (
                         MultiLineString(
-                            [LineString(_merged_nodep[e]) for e in _merged_edges]
+                            [
+                                LineString(_merged_nodep[e])
+                                for e in _merged_edges
+                            ]
                         ).is_simple
                     )
                     if intersect:
@@ -514,8 +546,12 @@ def addLoops(
                     or (
                         inner_fraction > max_inner_fraction
                     )  # if high fraction of points inside
-                    or (not np.isfinite(inner_fraction))  # prevent no points error
-                    or (np.sum(idx_close) > max_n_points)  # if too many points inside
+                    or (
+                        not np.isfinite(inner_fraction)
+                    )  # prevent no points error
+                    or (
+                        np.sum(idx_close) > max_n_points
+                    )  # if too many points inside
                     or pp_compactness(cycle_2d) < min_compactness
                 ):  # if loop is very narrow
                     valid = False
@@ -534,7 +570,10 @@ def addLoops(
 
             # ---> valid cycle, compute graph energy
             else:
-                _merged_part, _merged_part_dist = elpigraph.src.core.PartitionData(
+                (
+                    _merged_part,
+                    _merged_part_dist,
+                ) = elpigraph.src.core.PartitionData(
                     X, _merged_nodep, 10 ** 6, SquaredX=SquaredX
                 )
                 proj = elpigraph.src.reporting.project_point_onto_graph(
@@ -550,7 +589,9 @@ def addLoops(
                 energies.append(MSE)
                 merged_edges.append(_merged_edges)
                 merged_nodep.append(_merged_nodep)
-                merged_part.append(np.where(np.isin(_merged_part.flat, cycle_edges))[0])
+                merged_part.append(
+                    np.where(np.isin(_merged_part.flat, cycle_edges))[0]
+                )
                 loop_edges.append(edges)
                 loop_nodep.append(nodep[2:])
                 loop_leaves.append([c, l])
@@ -584,14 +625,18 @@ def addLoops(
                     # ----- cycle test
                     G = nx.Graph(_merged_edges.tolist())
                     cycle_edges = find_all_cycles(G)[0]
-                    cycle_nodep = np.array([_merged_nodep[e] for e in cycle_edges])
+                    cycle_nodep = np.array(
+                        [_merged_nodep[e] for e in cycle_edges]
+                    )
                     cent_part, cent_dists = elpigraph.src.core.PartitionData(
                         X, _merged_nodep, 10 ** 6, SquaredX=SquaredX
                     )
                     cycle_points = np.isin(cent_part.flat, cycle_edges)
 
                     if X.shape[1] > 2:
-                        pca = PCA(n_components=2, svd_solver="arpack").fit(cycle_nodep)
+                        pca = PCA(n_components=2, svd_solver="arpack").fit(
+                            cycle_nodep
+                        )
                         cycle_2d = pca.transform(cycle_nodep)
                         X_cycle_2d = pca.transform(X[cycle_points])
                     else:
@@ -616,13 +661,19 @@ def addLoops(
                     if type(shrunk_cycle_2d) == MultiPolygon:
                         in_shrunk_cycle = np.ones(len(X_inside), dtype=bool)
                     else:
-                        shrunk_cycle_2d = np.array(shrunk_cycle_2d.exterior.coords)
+                        shrunk_cycle_2d = np.array(
+                            shrunk_cycle_2d.exterior.coords
+                        )
 
                         # prevent bug when self-intersection
                         if len(shrunk_cycle_2d) == 0:
-                            in_shrunk_cycle = np.ones(len(X_inside), dtype=bool)
+                            in_shrunk_cycle = np.ones(
+                                len(X_inside), dtype=bool
+                            )
                         else:
-                            in_shrunk_cycle = in_hull(shrunk_cycle_2d, X_inside)
+                            in_shrunk_cycle = in_hull(
+                                shrunk_cycle_2d, X_inside
+                            )
                     idx_close = in_shrunk_cycle | (w < 1)
                     w = 1 - w / w.max()
                     w[idx_close] = 1
@@ -631,7 +682,9 @@ def addLoops(
                     compactness = pp_compactness(cycle_2d)
 
                     plt.title(
-                        f"{c}, {l}, MSE={MSE:.4f}, \n inner%={inner_fraction:.2f}, compactness={compactness:.2f}"
+                        f"{c}, {l}, MSE={MSE:.4f}, \n"
+                        f" inner%={inner_fraction:.2f},"
+                        f" compactness={compactness:.2f}"
                     )
                     plt.scatter(*X[:, :2].T, alpha=0.1, s=5)
                     plt.scatter(*X_fit[:, :2].T, s=5)
@@ -643,7 +696,9 @@ def addLoops(
                             c="k",
                         )
 
-                    _ = plt.scatter(*X[cycle_points][inside_idx, :2].T, c=w.flat, s=5)
+                    _ = plt.scatter(
+                        *X[cycle_points][inside_idx, :2].T, c=w.flat, s=5
+                    )
                     plt.colorbar(_)
 
                     plt.show()
@@ -656,7 +711,10 @@ def addLoops(
                 len(np.intersect1d(new_part[i], new_part[j]))
                 / max(len(new_part[i]), len(new_part[j]))
             ) > (2 / 3):
-                if np.argmin([new_inner_fraction[i], new_inner_fraction[j]]) == 0:
+                if (
+                    np.argmin([new_inner_fraction[i], new_inner_fraction[j]])
+                    == 0
+                ):
                     valid[i] = 0
                 else:
                     valid[j] = 0
@@ -666,7 +724,9 @@ def addLoops(
     new_leaves = [e for i, e in enumerate(new_leaves) if valid[i]]
     new_part = [e for i, e in enumerate(new_part) if valid[i]]
     new_energy = [e for i, e in enumerate(new_energy) if valid[i]]
-    new_inner_fraction = [e for i, e in enumerate(new_inner_fraction) if valid[i]]
+    new_inner_fraction = [
+        e for i, e in enumerate(new_inner_fraction) if valid[i]
+    ]
 
     ### form graph with all valid loops found ###
     if (new_edges == []) or (sum(valid) == 0):
@@ -676,12 +736,16 @@ def addLoops(
 
     for i, loop_edges in enumerate(new_edges):
         if i == 0:
-            loop_edges[(loop_edges != 0) & (loop_edges != 1)] += init_edges.max() - 1
+            loop_edges[(loop_edges != 0) & (loop_edges != 1)] += (
+                init_edges.max() - 1
+            )
             loop_edges[loop_edges == 0] = new_leaves[i][0]
             loop_edges[loop_edges == 1] = new_leaves[i][1]
             merged_edges = np.concatenate((init_edges, loop_edges))
         else:
-            loop_edges[(loop_edges != 0) & (loop_edges != 1)] += merged_edges.max() - 1
+            loop_edges[(loop_edges != 0) & (loop_edges != 1)] += (
+                merged_edges.max() - 1
+            )
             loop_edges[loop_edges == 0] = new_leaves[i][0]
             loop_edges[loop_edges == 1] = new_leaves[i][1]
             merged_edges = np.concatenate((merged_edges, loop_edges))
@@ -689,7 +753,9 @@ def addLoops(
 
     ### optionally refit the entire graph ###
     if fit_loops:
-        cycle_edges = np.concatenate(find_all_cycles(nx.Graph(merged_edges.tolist())))
+        cycle_edges = np.concatenate(
+            find_all_cycles(nx.Graph(merged_edges.tolist()))
+        )
         Mus = np.repeat(Mu, len(merged_nodep))
         Mus[cycle_edges] = Mu / 10000
         ElasticMatrix = elpigraph.src.core.Encode2ElasticMatrix(
@@ -704,7 +770,11 @@ def addLoops(
             _,
             _,
         ) = elpigraph.src.core.PrimitiveElasticGraphEmbedment(
-            X, merged_nodep, ElasticMatrix, PointWeights=weights, FixNodesAtPoints=[]
+            X,
+            merged_nodep,
+            ElasticMatrix,
+            PointWeights=weights,
+            FixNodesAtPoints=[],
         )
         # check intersection
         if merged_nodep.shape[1] == 2:
