@@ -48,6 +48,7 @@ def computeElasticPrincipalCircle(
     DensityRadius=None,
     AvoidSolitary=False,
     EmbPointProb=1,
+    PointWeights=None,
     SampleIC=True,
     AvoidResampling=True,
     AdjustElasticMatrix=None,
@@ -57,6 +58,17 @@ def computeElasticPrincipalCircle(
     DisplayWarnings=False,
     StoreGraphEvolution=False,
     GPU=False,
+    FixNodesAtPoints=[],
+    pseudotime=None,
+    pseudotimeLambda=0.01,
+    label=None,
+    labelLambda=0.01,
+    MaxNumberOfGraphCandidatesDict={
+        "AddNode2Node": float("inf"),
+        "BisectEdge": float("inf"),
+        "RemoveNode": float("inf"),
+        "ShrinkEdge": float("inf"),
+    },
 ):
 
     """
@@ -72,7 +84,7 @@ def computeElasticPrincipalCircle(
     #' @param InitNodes integer, number of points to include in the initial graph
     #' @param MaxNumberOfIterations integer, maximum number of steps to embed the nodes in the data
     #' @param TrimmingRadius real, maximal distance of point from a node to affect its embedment
-    #' @param eps real, minimal relative change in the position of the nodes to stop embedment 
+    #' @param eps real, minimal relative change in the position of the nodes to stop embedment
     #' @param Do_PCA boolean, should data and initial node positions be PCA trnasformed?
     #' @param InitNodePositions numerical 2D matrix, the k-by-m matrix with k m-dimensional positions of the nodes
     #' in the initial step
@@ -87,11 +99,11 @@ def computeElasticPrincipalCircle(
     #' @param drawAccuracyComplexity boolean, should the accuracy VS complexity plot be reported?
     #' @param drawPCAView boolean, should a 2D plot of the points and pricipal curve be dranw for the final configuration?
     #' @param drawEnergy boolean, should changes of evergy VS the number of nodes be reported?
-    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or 
+    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or
     #' cluster structure returned, e.g., by makeCluster. If a cluster structure is used, all the nodes must contains X
     #' (this is done using clusterExport)
     #' @param MinParOP integer, the minimum number of operations to use parallel computation
-    #' @param nReps integer, number of replica of the construction 
+    #' @param nReps integer, number of replica of the construction
     #' @param ProbPoint real between 0 and 1, probability of inclusing of a single point for each computation
     #' @param Subsets list of column names (or column number). When specified a principal circle will be computed for each of the subsets specified.
     #' @param NumEdges integer, the maximum nulber of edges
@@ -117,7 +129,7 @@ def computeElasticPrincipalCircle(
     #' @param ParallelRep boolean, should parallel execution be performed on the sampling instead of the the grammar evaluations?
     #' @param AvoidResampling booleand, should the sampling of initial conditions avoid reselecting the same points
     #' (or points neighbors if DensityRadius is specified)?
-    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable? 
+    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable?
     #' @param AdjustElasticMatrix.Initial a penalization function to adjust the elastic matrices of the initial configuration (e.g., AdjustByConstant).
     #' If NULL (the default), no penalization will be used.
     #' @param Lambda.Initial real, the lambda parameter used the construct the elastic matrix associted with ther initial configuration if needed.
@@ -127,7 +139,7 @@ def computeElasticPrincipalCircle(
     #'
     #' @return
     #'
-    #' A named list 
+    #' A named list
     #'
     #' @export
     #'
@@ -177,6 +189,7 @@ def computeElasticPrincipalCircle(
         InitNodePositions=InitNodePositions,
         InitEdges=InitEdges,
         AdjustVect=AdjustVect,
+        ElasticMatrix=ElasticMatrix,
         Configuration=Configuration,
         CenterData=CenterData,
         ComputeMSEP=ComputeMSEP,
@@ -200,6 +213,7 @@ def computeElasticPrincipalCircle(
         DensityRadius=DensityRadius,
         AvoidSolitary=AvoidSolitary,
         EmbPointProb=EmbPointProb,
+        PointWeights=PointWeights,
         SampleIC=SampleIC,
         AvoidResampling=AvoidResampling,
         #                                              ParallelRep = ParallelRep,
@@ -211,6 +225,12 @@ def computeElasticPrincipalCircle(
         MaxSteps=MaxSteps,
         StoreGraphEvolution=StoreGraphEvolution,
         GPU=GPU,
+        FixNodesAtPoints=FixNodesAtPoints,
+        pseudotime=pseudotime,
+        pseudotimeLambda=pseudotimeLambda,
+        label=label,
+        labelLambda=labelLambda,
+        MaxNumberOfGraphCandidatesDict=MaxNumberOfGraphCandidatesDict,
     )
 
 
@@ -248,8 +268,8 @@ def computeElasticPrincipalTree(
     Subsets=list(),
     ProbPoint=1,
     Mode=1,
-    FinalEnergy="Base",
-    alpha=0,
+    FinalEnergy="Penalized",
+    alpha=0.01,
     beta=0,
     # gamma = 0,
     # FastSolve = False,
@@ -258,6 +278,7 @@ def computeElasticPrincipalTree(
     DensityRadius=None,
     AvoidSolitary=False,
     EmbPointProb=1,
+    PointWeights=None,
     SampleIC=True,
     AvoidResampling=True,
     AdjustElasticMatrix=None,
@@ -267,6 +288,17 @@ def computeElasticPrincipalTree(
     DisplayWarnings=False,
     StoreGraphEvolution=False,
     GPU=False,
+    FixNodesAtPoints=[],
+    pseudotime=None,
+    pseudotimeLambda=0.01,
+    label=None,
+    labelLambda=0.01,
+    MaxNumberOfGraphCandidatesDict={
+        "AddNode2Node": float("inf"),
+        "BisectEdge": float("inf"),
+        "RemoveNode": float("inf"),
+        "ShrinkEdge": float("inf"),
+    },
 ):
     """
     #' Construct a principal elastic tree
@@ -281,7 +313,7 @@ def computeElasticPrincipalTree(
     #' @param InitNodes integer, number of points to include in the initial graph
     #' @param MaxNumberOfIterations integer, maximum number of steps to embed the nodes in the data
     #' @param TrimmingRadius real, maximal distance of point from a node to affect its embedment
-    #' @param eps real, minimal relative change in the position of the nodes to stop embedment 
+    #' @param eps real, minimal relative change in the position of the nodes to stop embedment
     #' @param Do_PCA boolean, should data and initial node positions be PCA trnasformed?
     #' @param InitNodePositions numerical 2D matrix, the k-by-m matrix with k m-dimensional positions of the nodes
     #' in the initial step
@@ -296,11 +328,11 @@ def computeElasticPrincipalTree(
     #' @param drawAccuracyComplexity boolean, should the accuracy VS complexity plot be reported?
     #' @param drawPCAView boolean, should a 2D plot of the points and pricipal curve be dranw for the final configuration?
     #' @param drawEnergy boolean, should changes of evergy VS the number of nodes be reported?
-    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or 
+    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or
     #' cluster structure returned, e.g., by makeCluster. If a cluster structure is used, all the nodes must contains X
     #' (this is done using clusterExport)
     #' @param MinParOP integer, the minimum number of operations to use parallel computation
-    #' @param nReps integer, number of replica of the construction 
+    #' @param nReps integer, number of replica of the construction
     #' @param ProbPoint real between 0 and 1, probability of inclusing of a single point for each computation
     #' @param Subsets list of column names (or column number). When specified a principal tree will be computed for each of the subsets specified.
     #' @param NumEdges integer, the maximum nulber of edges
@@ -326,7 +358,7 @@ def computeElasticPrincipalTree(
     #' @param ParallelRep boolean, should parallel execution be performed on the sampling instead of the the grammar evaluations?
     #' @param AvoidResampling booleand, should the sampling of initial conditions avoid reselecting the same points
     #' (or points neighbors if DensityRadius is specified)?
-    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable? 
+    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable?
     #' @param AdjustElasticMatrix.Initial a penalization function to adjust the elastic matrices of the initial configuration (e.g., AdjustByConstant).
     #' If NULL (the default), no penalization will be used.
     #' @param Lambda.Initial real, the lambda parameter used the construct the elastic matrix associted with ther initial configuration if needed.
@@ -337,7 +369,7 @@ def computeElasticPrincipalTree(
     #' @return A list of principal graph strucutures containing the trees constructed during the different replica of the algorithm.
     #' If the number of replicas is larger than 1. The the final element of the list is the "average tree", which is constructed by
     #' fitting the coordinates of the nodes of the reconstructed trees
-    #' @export 
+    #' @export
     #'
     #' @examples
     #'
@@ -382,6 +414,7 @@ def computeElasticPrincipalTree(
         InitNodePositions=InitNodePositions,
         InitEdges=InitEdges,
         AdjustVect=AdjustVect,
+        ElasticMatrix=ElasticMatrix,
         Configuration=Configuration,
         CenterData=CenterData,
         ComputeMSEP=ComputeMSEP,
@@ -405,6 +438,7 @@ def computeElasticPrincipalTree(
         DensityRadius=DensityRadius,
         AvoidSolitary=AvoidSolitary,
         EmbPointProb=EmbPointProb,
+        PointWeights=PointWeights,
         SampleIC=SampleIC,
         AvoidResampling=AvoidResampling,
         #                                              ParallelRep = ParallelRep,
@@ -416,6 +450,12 @@ def computeElasticPrincipalTree(
         MaxSteps=MaxSteps,
         StoreGraphEvolution=StoreGraphEvolution,
         GPU=GPU,
+        FixNodesAtPoints=FixNodesAtPoints,
+        pseudotime=pseudotime,
+        pseudotimeLambda=pseudotimeLambda,
+        label=label,
+        labelLambda=labelLambda,
+        MaxNumberOfGraphCandidatesDict=MaxNumberOfGraphCandidatesDict,
     )
 
 
@@ -463,6 +503,7 @@ def computeElasticPrincipalCurve(
     DensityRadius=None,
     AvoidSolitary=False,
     EmbPointProb=1,
+    PointWeights=None,
     SampleIC=True,
     AvoidResampling=True,
     AdjustElasticMatrix=None,
@@ -472,9 +513,20 @@ def computeElasticPrincipalCurve(
     DisplayWarnings=False,
     StoreGraphEvolution=False,
     GPU=False,
+    FixNodesAtPoints=[],
+    pseudotime=None,
+    pseudotimeLambda=0.01,
+    label=None,
+    labelLambda=0.01,
+    MaxNumberOfGraphCandidatesDict={
+        "AddNode2Node": float("inf"),
+        "BisectEdge": float("inf"),
+        "RemoveNode": float("inf"),
+        "ShrinkEdge": float("inf"),
+    },
 ):
 
-    """ 
+    """
     #' Construct a princial elastic curve
     #'
     #' This function is a wrapper to the computeElasticPrincipalGraph function that constructs the appropriate initial graph and grammars
@@ -487,7 +539,7 @@ def computeElasticPrincipalCurve(
     #' @param InitNodes integer, number of points to include in the initial graph
     #' @param MaxNumberOfIterations integer, maximum number of steps to embed the nodes in the data
     #' @param TrimmingRadius real, maximal distance of point from a node to affect its embedment
-    #' @param eps real, minimal relative change in the position of the nodes to stop embedment 
+    #' @param eps real, minimal relative change in the position of the nodes to stop embedment
     #' @param Do_PCA boolean, should data and initial node positions be PCA trnasformed?
     #' @param InitNodePositions numerical 2D matrix, the k-by-m matrix with k m-dimensional positions of the nodes
     #' in the initial step
@@ -502,11 +554,11 @@ def computeElasticPrincipalCurve(
     #' @param drawAccuracyComplexity boolean, should the accuracy VS complexity plot be reported?
     #' @param drawPCAView boolean, should a 2D plot of the points and pricipal curve be dranw for the final configuration?
     #' @param drawEnergy boolean, should changes of evergy VS the number of nodes be reported?
-    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or 
+    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or
     #' cluster structure returned, e.g., by makeCluster. If a cluster structure is used, all the nodes must contains X
     #' (this is done using clusterExport)
     #' @param MinParOP integer, the minimum number of operations to use parallel computation
-    #' @param nReps integer, number of replica of the construction 
+    #' @param nReps integer, number of replica of the construction
     #' @param ProbPoint real between 0 and 1, probability of inclusing of a single point for each computation
     #' @param Subsets list of column names (or column number). When specified a principal curve will be computed for each of the subsets specified.
     #' @param NumEdges integer, the maximum nulber of edges
@@ -538,12 +590,12 @@ def computeElasticPrincipalCurve(
     #' @param ParallelRep boolean, should parallel execution be performed on the sampling instead of the the grammar evaluations?
     #' @param AvoidResampling booleand, should the sampling of initial conditions avoid reselecting the same points
     #' (or points neighbors if DensityRadius is specified)?
-    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable? 
+    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable?
     #'
     #' @return A list of principal graph strucutures containing the curves constructed during the different replica of the algorithm.
     #' If the number of replicas is larger than 1. The the final element of the list is the "average curve", which is constructed by
     #' fitting the coordinates of the nodes of the reconstructed curves
-    #' @export 
+    #' @export
     #'
     #' @examples
     #'
@@ -586,6 +638,7 @@ def computeElasticPrincipalCurve(
         InitNodePositions=InitNodePositions,
         InitEdges=InitEdges,
         AdjustVect=AdjustVect,
+        ElasticMatrix=ElasticMatrix,
         Configuration=Configuration,
         CenterData=CenterData,
         ComputeMSEP=ComputeMSEP,
@@ -609,6 +662,7 @@ def computeElasticPrincipalCurve(
         DensityRadius=DensityRadius,
         AvoidSolitary=AvoidSolitary,
         EmbPointProb=EmbPointProb,
+        PointWeights=PointWeights,
         SampleIC=SampleIC,
         AvoidResampling=AvoidResampling,
         #                                              ParallelRep = ParallelRep,
@@ -620,6 +674,12 @@ def computeElasticPrincipalCurve(
         MaxSteps=MaxSteps,
         StoreGraphEvolution=StoreGraphEvolution,
         GPU=GPU,
+        FixNodesAtPoints=FixNodesAtPoints,
+        pseudotime=pseudotime,
+        pseudotimeLambda=pseudotimeLambda,
+        label=label,
+        labelLambda=labelLambda,
+        MaxNumberOfGraphCandidatesDict=MaxNumberOfGraphCandidatesDict,
     )
 
 
@@ -667,6 +727,7 @@ def fineTuneBR(
     DensityRadius=None,
     AvoidSolitary=False,
     EmbPointProb=1,
+    PointWeights=None,
     SampleIC=True,
     AvoidResampling=True,
     AdjustElasticMatrix=None,
@@ -676,6 +737,17 @@ def fineTuneBR(
     DisplayWarnings=False,
     StoreGraphEvolution=False,
     GPU=False,
+    FixNodesAtPoints=[],
+    pseudotime=None,
+    pseudotimeLambda=0.01,
+    label=None,
+    labelLambda=0.01,
+    MaxNumberOfGraphCandidatesDict={
+        "AddNode2Node": float("inf"),
+        "BisectEdge": float("inf"),
+        "RemoveNode": float("inf"),
+        "ShrinkEdge": float("inf"),
+    },
 ):
 
     """
@@ -691,7 +763,7 @@ def fineTuneBR(
     #' @param InitNodes integer, number of points to include in the initial graph
     #' @param MaxNumberOfIterations integer, maximum number of steps to embed the nodes in the data
     #' @param TrimmingRadius real, maximal distance of point from a node to affect its embedment
-    #' @param eps real, minimal relative change in the position of the nodes to stop embedment 
+    #' @param eps real, minimal relative change in the position of the nodes to stop embedment
     #' @param Do_PCA boolean, should data and initial node positions be PCA trnasformed?
     #' @param InitNodePositions numerical 2D matrix, the k-by-m matrix with k m-dimensional positions of the nodes
     #' in the initial step
@@ -706,11 +778,11 @@ def fineTuneBR(
     #' @param drawAccuracyComplexity boolean, should the accuracy VS complexity plot be reported?
     #' @param drawPCAView boolean, should a 2D plot of the points and pricipal curve be dranw for the final configuration?
     #' @param drawEnergy boolean, should changes of evergy VS the number of nodes be reported?
-    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or 
+    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or
     #' cluster structure returned, e.g., by makeCluster. If a cluster structure is used, all the nodes must contains X
     #' (this is done using clusterExport)
     #' @param MinParOP integer, the minimum number of operations to use parallel computation
-    #' @param nReps integer, number of replica of the construction 
+    #' @param nReps integer, number of replica of the construction
     #' @param ProbPoint real between 0 and 1, probability of inclusing of a single point for each computation
     #' @param Subsets list of column names (or column number). When specified a principal curve will be computed for each of the subsets specified.
     #' @param NumEdges integer, the maximum nulber of edges
@@ -739,13 +811,13 @@ def fineTuneBR(
     #' If NULL, the value of Lambda will be used.
     #' @param Mu.Initial real, the mu parameter used the construct the elastic matrix associted with ther initial configuration if needed.
     #' If NULL, the value of Mu will be used.
-    #' @param ParallelRep 
-    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable? 
+    #' @param ParallelRep
+    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable?
     #'
     #' @return A list of principal graph strucutures containing the curves constructed during the different replica of the algorithm.
     #' If the number of replicas is larger than 1. The the final element of the list is the "average curve", which is constructed by
     #' fitting the coordinates of the nodes of the reconstructed curves
-    #' @export 
+    #' @export
     #'
     #' @examples
     #'
@@ -803,6 +875,7 @@ def fineTuneBR(
         InitNodePositions=InitNodePositions,
         InitEdges=InitEdges,
         AdjustVect=AdjustVect,
+        ElasticMatrix=ElasticMatrix,
         Configuration=Configuration,
         CenterData=CenterData,
         ComputeMSEP=ComputeMSEP,
@@ -826,6 +899,7 @@ def fineTuneBR(
         DensityRadius=DensityRadius,
         AvoidSolitary=AvoidSolitary,
         EmbPointProb=EmbPointProb,
+        PointWeights=PointWeights,
         SampleIC=SampleIC,
         AvoidResampling=AvoidResampling,
         #                                              ParallelRep = ParallelRep,
@@ -836,6 +910,12 @@ def fineTuneBR(
         DisplayWarnings=DisplayWarnings,
         StoreGraphEvolution=StoreGraphEvolution,
         GPU=GPU,
+        FixNodesAtPoints=FixNodesAtPoints,
+        pseudotime=pseudotime,
+        pseudotimeLambda=pseudotimeLambda,
+        label=label,
+        labelLambda=labelLambda,
+        MaxNumberOfGraphCandidatesDict=MaxNumberOfGraphCandidatesDict,
     )
 
 
@@ -881,6 +961,7 @@ def GrowLeaves(
     DensityRadius=None,
     AvoidSolitary=False,
     EmbPointProb=1,
+    PointWeights=None,
     SampleIC=True,
     AvoidResampling=True,
     AdjustElasticMatrix=None,
@@ -890,6 +971,17 @@ def GrowLeaves(
     DisplayWarnings=False,
     StoreGraphEvolution=False,
     GPU=False,
+    FixNodesAtPoints=[],
+    pseudotime=None,
+    pseudotimeLambda=0.01,
+    label=None,
+    labelLambda=0.01,
+    MaxNumberOfGraphCandidatesDict={
+        "AddNode2Node": float("inf"),
+        "BisectEdge": float("inf"),
+        "RemoveNode": float("inf"),
+        "ShrinkEdge": float("inf"),
+    },
 ):
 
     """
@@ -905,7 +997,7 @@ def GrowLeaves(
     #' @param InitNodes integer, number of points to include in the initial graph
     #' @param MaxNumberOfIterations integer, maximum number of steps to embed the nodes in the data
     #' @param TrimmingRadius real, maximal distance of point from a node to affect its embedment
-    #' @param eps real, minimal relative change in the position of the nodes to stop embedment 
+    #' @param eps real, minimal relative change in the position of the nodes to stop embedment
     #' @param Do_PCA boolean, should data and initial node positions be PCA trnasformed?
     #' @param InitNodePositions numerical 2D matrix, the k-by-m matrix with k m-dimensional positions of the nodes
     #' in the initial step
@@ -920,11 +1012,11 @@ def GrowLeaves(
     #' @param drawAccuracyComplexity boolean, should the accuracy VS complexity plot be reported?
     #' @param drawPCAView boolean, should a 2D plot of the points and pricipal curve be dranw for the final configuration?
     #' @param drawEnergy boolean, should changes of evergy VS the number of nodes be reported?
-    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or 
+    #' @param n.cores either an integer (indicating the number of cores to used for the creation of a cluster) or
     #' cluster structure returned, e.g., by makeCluster. If a cluster structure is used, all the nodes must contains X
     #' (this is done using clusterExport)`
     #' @param MinParOP integer, the minimum number of operations to use parallel computation
-    #' @param nReps integer, number of replica of the construction 
+    #' @param nReps integer, number of replica of the construction
     #' @param ProbPoint real between 0 and 1, probability of inclusing of a single point for each computation
     #' @param Subsets list of column names (or column number). When specified a principal curve will be computed for each of the subsets specified.
     #' @param NumEdges integer, the maximum nulber of edges
@@ -953,13 +1045,13 @@ def GrowLeaves(
     #' If NULL, the value of Lambda will be used.
     #' @param Mu.Initial real, the mu parameter used the construct the elastic matrix associted with ther initial configuration if needed.
     #' If NULL, the value of Mu will be used.
-    #' @param ParallelRep 
-    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable? 
+    #' @param ParallelRep
+    #' @param SampleIC boolean, should the initial configuration be considered on the sampled points when applicable?
     #'
     #' @return A list of principal graph strucutures containing the curves constructed during the different replica of the algorithm.
     #' If the number of replicas is larger than 1. The the final element of the list is the "average curve", which is constructed by
     #' fitting the coordinates of the nodes of the reconstructed curves
-    #' @export 
+    #' @export
     #'
     #' @examples
     #'
@@ -995,7 +1087,7 @@ def GrowLeaves(
         InitNodes=InitNodes,
         Lambda=Lambda,
         Mu=Mu,
-        GrowGrammars=np.array([["addnodetonode_1"]]),
+        GrowGrammars=np.array([["addnode2node_1"]]),
         ShrinkGrammars=np.array([]),
         GrammarOrder=["Grow"],
         GrammarOptimization=False,
@@ -1007,6 +1099,7 @@ def GrowLeaves(
         InitNodePositions=InitNodePositions,
         InitEdges=InitEdges,
         AdjustVect=AdjustVect,
+        ElasticMatrix=ElasticMatrix,
         Configuration=Configuration,
         CenterData=CenterData,
         ComputeMSEP=ComputeMSEP,
@@ -1030,6 +1123,7 @@ def GrowLeaves(
         DensityRadius=DensityRadius,
         AvoidSolitary=AvoidSolitary,
         EmbPointProb=EmbPointProb,
+        PointWeights=PointWeights,
         SampleIC=SampleIC,
         AvoidResampling=AvoidResampling,
         #                                              ParallelRep = ParallelRep,
@@ -1040,6 +1134,12 @@ def GrowLeaves(
         DisplayWarnings=DisplayWarnings,
         StoreGraphEvolution=StoreGraphEvolution,
         GPU=GPU,
+        FixNodesAtPoints=FixNodesAtPoints,
+        pseudotime=pseudotime,
+        pseudotimeLambda=pseudotimeLambda,
+        label=label,
+        labelLambda=labelLambda,
+        MaxNumberOfGraphCandidatesDict=MaxNumberOfGraphCandidatesDict,
     )
 
 
@@ -1052,8 +1152,9 @@ def generateInitialConfiguration(
     PCADensity=True,
     CenterDataDensity=True,
     verbose=False,
+    FixNodesAtPoints=[],
 ):
-    """  
+    """
     #' Produce an initial graph with a given structure
     #'
     #' @param X numerical 2D matrix, the n-by-m matrix with the position of n m-dimensional points
@@ -1074,7 +1175,7 @@ def generateInitialConfiguration(
     #' @param DensityRadius numeric, the radius used to estimate local density. This need to be set when Configuration is equal to "Density"
     #' @param MaxPoints integer, the maximum number of points for which the local density will be estimated. If the number of data points is
     #' larger than MaxPoints, a subset of the original points will be sampled
-    #' @param PCADensity boolean, should PCA be applied to the data before computing the most dense area 
+    #' @param PCADensity boolean, should PCA be applied to the data before computing the most dense area
     #'
     #' @return
     #' @export
@@ -1095,9 +1196,9 @@ def generateInitialConfiguration(
         st = np.std(PC1, ddof=1)
         NodeP = np.dot(np.linspace(mn - st, mn + st, Nodes)[:, None], Vt)
         NodePositions = NodeP + mv[None]
+
         # Creating edges
         edges = np.vstack((np.arange(Nodes - 1), np.arange(1, Nodes))).T
-
         DONE = True
 
     if Configuration == "Circle":
@@ -1154,7 +1255,12 @@ def generateInitialConfiguration(
 
         ID1 = np.random.choice(len(X))
 
-        Dist = PartialDistance(X[[ID1],], X).flatten()
+        Dist = PartialDistance(
+            X[
+                [ID1],
+            ],
+            X,
+        ).flatten()
         with np.errstate(divide="ignore"):
             Probs = 1 / (Dist)
         if DensityRadius:
@@ -1184,7 +1290,12 @@ def generateInitialConfiguration(
         RandClus = np.random.choice(10)
         ID1 = np.random.choice(np.where(KM.labels_ == RandClus)[0])
 
-        Dist = PartialDistance(X[[ID1],], X).flatten()
+        Dist = PartialDistance(
+            X[
+                [ID1],
+            ],
+            X,
+        ).flatten()
 
         with np.errstate(divide="ignore"):
             Probs = 1 / (Dist)
@@ -1229,7 +1340,14 @@ def generateInitialConfiguration(
 
             SampledIdxs = np.random.choice(len(tX), size=MaxPoints)
 
-            PartStruct = PartialDistance(tX[SampledIdxs,], tX[SampledIdxs,])
+            PartStruct = PartialDistance(
+                tX[
+                    SampledIdxs,
+                ],
+                tX[
+                    SampledIdxs,
+                ],
+            )
             PointsInNei = np.sum(PartStruct < DensityRadius, axis=1)
 
             if max(PointsInNei) < 2:
@@ -1242,7 +1360,13 @@ def generateInitialConfiguration(
             NodePositions = X[
                 SampledIdxs[
                     np.random.choice(
-                        np.where(PartStruct[IdMax,] < DensityRadius)[0], size=2
+                        np.where(
+                            PartStruct[
+                                IdMax,
+                            ]
+                            < DensityRadius
+                        )[0],
+                        size=2,
                     )
                 ],
                 :,
@@ -1265,7 +1389,13 @@ def generateInitialConfiguration(
             IdMax = np.argmax(PointsInNei)
             NodePositions = X[
                 np.random.choice(
-                    np.where(PartStruct[IdMax,] < DensityRadius)[0], size=2
+                    np.where(
+                        PartStruct[
+                            IdMax,
+                        ]
+                        < DensityRadius
+                    )[0],
+                    size=2,
                 ),
                 :,
             ]
@@ -1305,7 +1435,14 @@ def generateInitialConfiguration(
 
             SampledIdxs = np.random.choice(len(tX), size=MaxPoints)
 
-            PartStruct = PartialDistance(tX[SampledIdxs,], tX[SampledIdxs,])
+            PartStruct = PartialDistance(
+                tX[
+                    SampledIdxs,
+                ],
+                tX[
+                    SampledIdxs,
+                ],
+            )
             PointsInNei = np.sum(PartStruct < DensityRadius, axis=1)
 
             if max(PointsInNei) < 2:
@@ -1318,7 +1455,13 @@ def generateInitialConfiguration(
 
             NodePositions = X[
                 np.random.choice(
-                    np.where(PartStruct[IdMax,] < DensityRadius)[0], size=2
+                    np.where(
+                        PartStruct[
+                            IdMax,
+                        ]
+                        < DensityRadius
+                    )[0],
+                    size=2,
                 ),
                 :,
             ]
@@ -1342,7 +1485,13 @@ def generateInitialConfiguration(
 
             NodePositions = X[
                 np.random.choice(
-                    np.where(PartStruct[IdMax,] < DensityRadius)[0], size=2
+                    np.where(
+                        PartStruct[
+                            IdMax,
+                        ]
+                        < DensityRadius
+                    )[0],
+                    size=2,
                 ),
                 :,
             ]
@@ -1356,4 +1505,3 @@ def generateInitialConfiguration(
         return dict(NodePositions=NodePositions, Edges=edges)
     else:
         raise ValueError("Unsupported configuration!")
-
